#!/usr/bin/env bash

# ----------------------------------------------------------- #
# Copyright (C) 2008 Red Hat, Inc.                            #
# Written by Michel Samia <msamia@redhat.com>                 #
# tcp_wrappers.sh                                             #
# more info in tcp_wrappers.dsc                               #
# ----------------------------------------------------------- #


if (( ${DEBUG} == 1 )); then 
. "${TESTS_DIRS}/bash_defs_debug.sh" 
else
. "${TESTS_DIRS}/bash_defs.sh"
fi

ALLOW='/etc/hosts.allow'
DENY='/etc/hosts.deny'

const W_NOPOLICY=1

# filter binaries which use specified library
# stdin: list of paths to binaries
# stdout: filtered list
# usage: filter_lib libname
function filter_lib {
	while read prog; do
		ldd "$prog" 2>/dev/null | awk '{ print $1 }' \
			| sed -e 's/\.so\(\.[0-9]\+\)\?//g' -e 's/^.*\///g' \
			| grep -F "$1" >/dev/null && echo "$prog"
	done
}

if (( ${REFRESH} == 1 )); then
    test_exit ${E_OK}
fi

# get list of applications, which are linked with libwrap
apps=`ls -1 $CHECK | filter_lib libwrap | grep -v config | xargs -n1 basename`
ps="`ps -eo comm=`"

echo "$apps" |
{
	while read app
	do

		# TODO: it would be better to grep it with sth like "^[^:#]*$app|^ALL *:" to work with lists of services on one line...

		# if the application is not running, we will not cry about it..
		echo "$ps" | grep "$app" >/dev/null || continue

		#report 'INFO' 1 "App: $app"

		egrep "^$app|^ALL *:" $ALLOW > /dev/null
		isInAllow=$[ 1 - $? ]

		egrep "^$app|^ALL *:" $DENY > /dev/null
		isInDeny=$[ 1 - $? ]
		
		if [ $isInAllow -ne 1 ] && [ $isInDeny -ne 1 ]
		then
			report 'WARNING' $W_NOPOLICY "Application $app is using tcp wrappers, but does not specify any restrictions in $ALLOW nor in $DENY"
			report 'HINT' $W_NOPOLICY "Please consider restricting access to this service i.e. according to IP address ranges"
		fi
		
	done
}
